%%    Minimal conductivity of graphene - based on EQuUs v4.4
%    Copyright (C) 2015 Peter Rakyta, Ph.D.
%
%    This program is free software: you can redistribute it and/or modify
%    it under the terms of the GNU General Public License as published by
%    the Free Software Foundation, either version 3 of the License, or
%    (at your option) any later version.
%
%    This program is distributed in the hope that it will be useful,
%    but WITHOUT ANY WARRANTY; without even the implied warranty of
%    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%    GNU General Public License for more details.
%
%    You should have received a copy of the GNU General Public License
%    along with this program.  If not, see http://www.gnu.org/licenses/.
%
% Calculates the minimal conductivity of the graphene sheet as a function of the aspect ratio.
function MinimalConductivity( filenum )

if ~exist('filenum', 'var')
    filenum = 1;
end

outfilename = [pwd, filesep, 'Ribbon_Conductivity',num2str( filenum ),'.mat'];
outputdir   = [];

height = 170;
Energy = 0.001; %In EV


ribbon_hamiltonian    = [];



Widthmin = 10;
Widthdelta = 20;
Widthnum = 30;%100;%200;
Widthvec = Widthmin:Widthdelta:Widthmin+Widthnum*Widthdelta;

% calculated conductance (and number of open channels) as a function of width/height
Conductivity = NaN(Widthnum+1,1); 
DeltaC      = NaN(Widthnum+1,1);
aspectRatio = NaN(Widthnum+1,1); 

setOutputDir();


CalculateTranspor();
Egeszabra();




%% Calculates transport
    function CalculateTranspor()
        
        
        
        for idx = 1:length(Widthvec)       
            width = Widthvec(idx);
            ribbon_hamiltonian = Ribbon('width', width, 'height', height );     
        
            disp(' ----------------------------------')
            disp('Plotting conductivity:')
            Egeszabra();        
        
        
            disp(' ')
            disp(' ')
            disp(' ----------------------------------')
            disp('Calculating Conductance for the given aspect ratio')           
        

            
            tic
            [Conductivity_tmp,aspectRatio_tmp,~,~,DeltaC_tmp] = ribbon_hamiltonian.Transport( Energy, 'constant_channels', true );
            toc
            
            Conductivity(idx) = Conductivity_tmp;
            DeltaC(idx) = DeltaC_tmp;
            aspectRatio(idx) = aspectRatio_tmp;
            
            
            
            save( [outfilename], 'Conductivity', 'aspectRatio', 'Widthvec', 'Energy', 'DeltaC', 'height' );
            disp([ num2str(idx/length(Widthvec)*100),' % calculated of the conductance.'])
            disp( ' ' )
            
            
            
        end
                
    end



%% sets the output directory
    function setOutputDir()
        resultsdir = [pwd, filesep, 'results'];
        mkdir(resultsdir );
        outputdir = resultsdir;        
    end




    function Egeszabra( )
        

        
        figure1 = figure();
        
        fontsize = 9;
        
        %% Conductivity    
        
        theoretical_limit = 2/pi; %Theoretical minimal conductivity
        
        
        indexek = logical( (abs(DeltaC) <= 0.5) & (aspectRatio~=0) & ~isnan(DeltaC) );
        x_lim = [0 max(aspectRatio(indexek))];
        
        if norm(x_lim) == 0
            x_lim = [0 1];
        end
        
        Position = [0.08 0.58 0.53 0.3];
        axes_cond = axes('Parent',figure1, 'Position', Position,...
                'Visible', 'on',...
                'FontSize', fontsize,... 
                'xlim', x_lim,...                    'ylim', y_lim,...                'XTick', XTick,...                'YTick', YTick,...
                'Box', 'on',...
                'FontName','Times New Roman');
        hold on; 
        
        
        numerics = plot(aspectRatio(indexek), Conductivity(indexek), 'Linewidth', 2, 'color', [0 0 0], 'Parent', axes_cond);   
        try
            plot(aspectRatio(indexek), DeltaC(indexek), 'Linewidth', 1, 'color', [0 1 0], 'Parent', axes_cond);
        catch
            disp('No DeltaC data present')
        end
         plot( [0 max(aspectRatio(indexek))], ones(1,2)*theoretical_limit, 'r',  'Parent', axes_cond);
        
               
         
        
         % Create xlabel
        xlabel('W/L','FontSize', fontsize,'FontName','Times New Roman', 'Parent', axes_cond);

        % Create ylabel
        ylabel('\sigma/\sigma_0','FontSize', fontsize,'FontName','Times New Roman', 'Parent', axes_cond);
        
        

        fig_legend = legend(axes_cond, {'conductivity', 'unitarity'});%, 'FontSize', fontsize, 'FontName','Times New Roman')
        set(fig_legend, 'FontSize', fontsize, 'FontName','Times New Roman', 'Box', 'off', 'Location', 'NorthEast')

        
        print('-depsc2', [outputdir,'/Minimal_Conductivity_', num2str(filenum),'.eps'])
        print('-dpdf', [outputdir,'/Minimal_Conductivity_', num2str(filenum),'.pdf'])
        close(figure1)
        
        
    end




end
