%    Magnetic transport through a quantum antidot  - based on EQuUs v4.7
%    Copyright (C) 2015 Peter Rakyta, Ph.D.
%
%    This program is free software: you can redistribute it and/or modify
%    it under the terms of the GNU General Public License as published by
%    the Free Software Foundation, either version 3 of the License, or
%    (at your option) any later version.
%
%    This program is distributed in the hope that it will be useful,
%    but WITHOUT ANY WARRANTY; without even the implied warranty of
%    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%    GNU General Public License for more details.
%
%    You should have received a copy of the GNU General Public License
%    along with this program.  If not, see http://www.gnu.org/licenses/.
function Graphene_antidot_transport( filenum )

if ~exist('filenum', 'var')
    filenum = 1;
end


toCalculate = 1;
outfilename = ['Hole_in_graphene_',num2str( filenum ),'.mat'];
outputdir   = [];


width = 220;
height = 350;
B = 40; %in Tesla


hole = struct( 'center', [], 'radius', [] );
hole.center = [width/2;height/2];
hole.radius = 90;

points = round([27 195 27 195]);
scatterers.zpoints = round(points + rand(size(points))*5);
scatterers.z = round( ones(size(points))*height + [-30 -30 30 30] + rand(size(points))*5);
scatterers.siteindexes = [];
scatterers.aremissing = 1;



spectrum = []; % calculated spectrum of the lead
homega = [];
hole_wavefnc = [];



create_hole           = [];
coordinates           = [];
hole_edge_points      = [];
hole_eigenenergies    = [];


Emin = 0.001;%0.0215;%0.015;
Emax = 0.22;%0.37;%0.2;
Enum = 200;%100;%200;
Evec = Emin:(Emax-Emin)/Enum:Emax;
Enum = length(Evec)-1;

% calculated conductance (and number of open channels) as a function of B
Conductance = zeros(Enum+1,1); 
nyitott_csatornak = zeros(Enum+1,1);
DeltaC      = zeros(Enum+1,1);


setOutputDir();



CalculateTranspor();
Egeszabra();




%% Calculates transport
    function CalculateTranspor()
        

        create_hole = antidot('width', width, 'height', height, 'B', B, 'hole', hole, 'scatterers', scatterers );
        create_hole.CreateHandlesForMagneticField();
        
        
       
%         disp(' ')
%         disp(' ')
%         disp(' ----------------------------------')
%         disp('Cerating Lead Hamiltonians:')          
%         create_hole.FL_handles.LeadCalc();
        
        
        disp(' ')
        disp(' ')
        disp(' ----------------------------------')
        disp('Plotting geometry and spectrum of leads:')
        coordinates = create_hole.coordinates;
        
        % calculate spectrum of teh lead
        %create_hole.CreateHandlesForMagneticField();
        create_hole.setEnergy( Evec(1) )
        create_hole.CreateRibbon( 'justHamiltonians', true );
        CalcSpektrum();
        create_hole.getHolePoints();
        hole_edge_points = create_hole.hole_edge_points;
        Egeszabra();        
        
        
        disp(' ')
        disp(' ')
        disp(' ----------------------------------')
        disp('Calculating Conductance for the given magnetic field') 
     
        
        
        for idx = 1:length(Evec)    
            tic
            [C,nyitott_csatornak_num,DeltaC_tmp] = create_hole.Transport( Evec(idx) );
            toc
            
            Conductance(idx) = C;
            nyitott_csatornak(idx) = nyitott_csatornak_num(1);
            DeltaC(idx) = DeltaC_tmp;
            
            
            save( [outputdir,'/',outfilename], 'Conductance', 'nyitott_csatornak', 'B', 'coordinates', 'Evec', 'hole', 'DeltaC', 'width', 'height', 'hole_edge_points', 'scatterers', 'hole_eigenenergies' );
            disp([ num2str(idx/length(Evec)*100),' % calculated of the conductance.'])
            disp( ' ' )
            
            Egeszabra();
            
            
        end
        
        toCalculate = false;
        Egeszabra();
        
    end


%% calculates the spectrum of the leads
    function CalcSpektrum()
        
    rCC = create_hole.rCC*1e-10; %Angstrom
    phi0 = create_hole.h/create_hole.qe; % magnetic flux quantum    
    eta_B = 2*pi/phi0*(rCC)^2*create_hole.B;
    homega = 2.97*sqrt(9/2*eta_B); 
    
    kmin = 1*sqrt(3);%1.4*sqrt(3);
    kmax = 1.8*sqrt(3);%3*sqrt(3);
    knum = 50;
    spectrum = create_hole.Surface_tmp.CalcSpektrum( 'toPlot', 0, 'ka_min', kmin, 'ka_max', kmax, 'ka_num', knum );
 
    end
 
 
%% Hamiltonian for the secular equation
    function H = secular_H(H0,H1,k)
        
        H = H0 + H1*exp(1i*k) + H1'*exp(-1i*k);
        
    end


%% sets output directory
    function setOutputDir()
        resultsdir = 'results';
        mkdir(resultsdir );
        outputdir = [ resultsdir ];        
    end




    function Egeszabra( )
        
        if toCalculate
            figure1 = figure();
        else
            figure1 = figure('XVisual','0x21 (TrueColor, depth 24, RGB mask 0xff0000 0xff00 0x00ff)','Renderer','painters',...
    'Colormap',[1 1 0.994791686534882;0.954365074634552 0.954365074634552 0.932043671607971;0.908730149269104 0.908730149269104 0.869295656681061;0.863095223903656 0.863095223903656 0.80654764175415;0.817460358142853 0.817460358142853 0.74379962682724;0.771825432777405 0.771825432777405 0.68105161190033;0.726190507411957 0.726190507411957 0.618303596973419;0.680555582046509 0.680555582046509 0.555555582046509;0.643518567085266 0.629629671573639 0.518518567085266;0.606481492519379 0.578703701496124 0.481481492519379;0.569444477558136 0.527777791023254 0.444444477558136;0.532407402992249 0.476851880550385 0.407407432794571;0.495370388031006 0.425925940275192 0.370370388031006;0.458333343267441 0.375 0.333333343267441;0.421296298503876 0.32407408952713 0.296296298503876;0.384259253740311 0.273148149251938 0.259259253740311;0.347222208976746 0.222222223877907 0.222222223877907;0.33983451128006 0.217494085431099 0.217494085431099;0.332446783781052 0.212765961885452 0.212765961885452;0.325059086084366 0.208037823438644 0.208037823438644;0.31767138838768 0.203309699892998 0.203309699892998;0.310283690690994 0.19858156144619 0.19858156144619;0.302895963191986 0.193853422999382 0.193853422999382;0.2955082654953 0.189125299453735 0.189125299453735;0.288120567798615 0.184397161006927 0.184397161006927;0.280732840299606 0.179669037461281 0.179669037461281;0.273345142602921 0.174940899014473 0.174940899014473;0.265957444906235 0.170212760567665 0.170212760567665;0.258569717407227 0.165484637022018 0.165484637022018;0.251182019710541 0.160756498575211 0.160756498575211;0.243794322013855 0.156028375029564 0.156028375029564;0.236406609416008 0.151300236582756 0.151300236582756;0.229018896818161 0.146572098135948 0.146572098135948;0.221631199121475 0.141843974590302 0.141843974590302;0.214243486523628 0.137115836143494 0.137115836143494;0.206855788826942 0.132387712597847 0.132387712597847;0.199468076229095 0.127659574151039 0.127659574151039;0.192080363631248 0.122931443154812 0.122931443154812;0.184692665934563 0.118203312158585 0.118203312158585;0.177304953336716 0.113475181162357 0.113475181162357;0.16991725564003 0.108747042715549 0.108747042715549;0.162529543042183 0.104018911719322 0.104018911719322;0.155141845345497 0.0992907807230949 0.0992907807230949;0.14775413274765 0.0945626497268677 0.0945626497268677;0.140366420149803 0.0898345187306404 0.0898345187306404;0.132978722453117 0.0851063802838326 0.0851063802838326;0.12559100985527 0.0803782492876053 0.0803782492876053;0.118203304708004 0.075650118291378 0.075650118291378;0.110815599560738 0.0709219872951508 0.0709219872951508;0.103427894413471 0.0661938562989235 0.0661938562989235;0.0960401818156242 0.0614657215774059 0.0614657215774059;0.0886524766683578 0.0567375905811787 0.0567375905811787;0.0812647715210915 0.0520094558596611 0.0520094558596611;0.0738770663738251 0.0472813248634338 0.0472813248634338;0.0664893612265587 0.0425531901419163 0.0425531901419163;0.059101652354002 0.037825059145689 0.037825059145689;0.0517139472067356 0.0330969281494617 0.0330969281494617;0.0443262383341789 0.0283687952905893 0.0283687952905893;0.0369385331869125 0.0236406624317169 0.0236406624317169;0.029550826177001 0.0189125295728445 0.0189125295728445;0.0221631191670895 0.0141843976452947 0.0141843976452947;0.0147754130885005 0.00945626478642225 0.00945626478642225;0.00738770654425025 0.00472813239321113 0.00472813239321113;0 0 0]);
        end

        fontsize = 9;
        
        %% geometry    
        deltax_ps = 2;  % 2n-1 es 2n kozotti tavolsag rCC egysegekben
        deltax_pt = 1;    % 2n es 2n+1 kozotti tavolsag rCC egysegekben
        deltax    = 3; % 2n-1 es 2n+1 kozotti tavolsag rCC egysegekben
        deltay = sqrt(3);   
        
        rightside = width/2*deltax*create_hole.rCC;
        top = height*deltay*create_hole.rCC;
        
        %x_lim = [min(coordinates.x(:,1)) max(coordinates.x(:,1)) ];
        %x_lim = [0 rightside];
        
        Position = [0.08 0.58 0.33 0.4];
        axes_geo = axes('Parent',figure1, 'Position', Position,...
                'CLim',[0 0.028],...
                'Visible', 'on',...
                'FontSize', fontsize,...                               'xlim', x_lim,...                 'ylim', y_lim,...                'XTick', XTick,...                'YTick', YTick,...
                'Box', 'on',...
                'FontName','Times New Roman');
        hold on; 
        axis equal;
        
        
        if ( toCalculate )
            for idx = 1:size(hole_edge_points,1)
    	        ycoord = deltay*hole_edge_points(idx,1)*ones(2,1)*create_hole.rCC;
            
    	        xcoord = zeros(2,1);
    	        if mod(hole_edge_points(idx,2),2) == 0
    	           xcoord(1) = (hole_edge_points(idx,2)-2)/2*deltax + deltax_ps;
    	        else
    	           xcoord(1) = (hole_edge_points(idx,2)-1)/2*deltax;
    	        end
            
    	        if mod(hole_edge_points(idx,2),3) == 0
    	           xcoord(2) = (hole_edge_points(idx,3)-2)/2*deltax + deltax_ps;
    	        else
    	           xcoord(2) = (hole_edge_points(idx,3)-1)/2*deltax;
    	        end  
    	        xcoord = xcoord*create_hole.rCC;
    		    plot(xcoord, ycoord,'+','MarkerSize', 4, 'Parent', axes_geo )  
            end  
 
    		    
        else
            
            if isempty(hole_wavefnc)
                Energy = 0.1566375;
                hole_wavefnc = create_hole.CalcWavefnc( Energy, 1, 'toPlot', 0, 'filterHole', 1, 'db', 1, 'infinitemass', 0, 'dotCalc', 0 );
            end            
	        delta = 1;
	        levels = 25;
            Wavefnc = hole_wavefnc.Wavefnc{1};
	        contourf( hole_wavefnc.xcoords(1:delta:end,1:delta:end), hole_wavefnc.ycoords(1:delta:end,1:delta:end), abs(Wavefnc(1:delta:end,1:delta:end)), ...
	    	    levels, 'LineStyle', 'none',  'Parent', axes_geo );  
	        text(30,80, ['E = ', num2str(hole_wavefnc.eigenval*1000,3),' meV'], 'color', [0,0,0], 'FontSize', fontsize, 'FontName','Times New Roman',  'Parent', axes_geo);
        end
        
	    plot([0,0], [0,top],'LineWidth', 4, 'color', [0 0 0], 'Parent', axes_geo )
	    plot([rightside,rightside], [0,top],'LineWidth', 4, 'color', [0 0 0], 'Parent', axes_geo )  

        
        if isfield(scatterers, 'aremissing')
            if scatterers.aremissing
               marker = 'o';
               markersize = 3;
            else
               marker = 'x';
               markersize = 5;
            end
        else
            marker = 'x';
            markersize = 5;
        end 
        plot( scatterers.zpoints/2*deltax*create_hole.rCC, scatterers.z/2*deltay*create_hole.rCC, ...
    		    marker, 'MarkerSize', markersize, 'Parent', axes_geo ) 
         
        
        
        % Create xlabel
        xlabel('x [A]','FontSize', fontsize,'FontName','Times New Roman', 'Parent', axes_geo);

        % Create ylabel
        ylabel('y [A]','FontSize', fontsize,'FontName','Times New Roman', 'Parent', axes_geo);
        
        Lead1_pos = [ mean(coordinates.x(:,1)), min(coordinates.y(1,:))+20];
        text(Lead1_pos(1), Lead1_pos(2), ['Lead'], 'FontSize', fontsize, 'FontName','Times New Roman',  'Parent', axes_geo);
        
        Lead2_pos = [ mean(coordinates.x(:,1)), max(coordinates.y(1,:))-20];
        text(Lead2_pos(1), Lead2_pos(2), ['Lead'], 'FontSize', fontsize, 'FontName','Times New Roman',  'Parent', axes_geo);
        

        
        
        
%% sepctrum in the lead        
        x_lim = [min(spectrum(:,1)) max(spectrum(:,1))]/sqrt(3);
        y_lim = [min(spectrum(:,2));max(spectrum(:,2))]*0.8;

        Position = [0.5 0.58 0.33 0.4];
        axes_spectrum = axes('Parent',figure1, 'Position', Position,...
                'Visible', 'on',...
                'FontSize', fontsize,...
                'xlim', x_lim,...
                'ylim', y_lim,...                'XTick', XTick,...                'YTick', YTick,...
                'Box', 'on',...
                'FontName','Times New Roman');
        hold on; 
        
        Kpoint = 2*pi/3/sqrt(3);
        Kpoint_pos = [Kpoint+0.005, 0.8*y_lim(2)];
        plot(spectrum(:,1)/sqrt(3), spectrum(:,2),'.', 'MarkerSize', 4, 'Parent', axes_spectrum )  
        plot(Kpoint*ones(2,1), [min(spectrum(:,2));max(spectrum(:,2))], 'LineWidth', 1, 'Color', [0 0 0],  'Parent', axes_spectrum )  
        text(Kpoint_pos(1), Kpoint_pos(2), 'K', 'FontSize', fontsize, 'FontName','Times New Roman',  'Parent', axes_spectrum);
        
        % Create xlabel
        xlabel_position = [0 0 0];
        xlabel('k_yr_{CC}','FontSize', fontsize,'FontName','Times New Roman', 'Parent', axes_spectrum);
        xlabel_handle = get(axes_spectrum,'XLabel');  
        set(xlabel_handle, 'Position', get(xlabel_handle, 'Position') + xlabel_position);

        % Create ylabel
        ylabel_position = [0 0 0];
        ylabel('E [eV]','FontSize', fontsize,'FontName','Times New Roman', 'Parent', axes_spectrum);
        ylabel_handle = get(axes_spectrum,'YLabel'); 
        set(ylabel_handle, 'Position', get(ylabel_handle, 'Position') + ylabel_position);
        
        % Create textbox
        annotation(figure1,'textbox',[0.65 0.7345 0.244 0.1274],...
    'String',{['h\omega = ',num2str(homega,'% 10.3f' ),'eV']},...
    'FontSize',fontsize,...
    'FontName','Times New Roman',...
    'LineStyle','none');

%% Wavefunction in the lead

        specific_k = 1.25*sqrt(3);
        
        Surface_temp = create_hole.Surface_tmp;%create_hole.CreateRibbon();
        H0 = Surface_temp.Read( 'H0' );
        H1 = Surface_temp.Read( 'H1' );
        H0 = H0 - eye(size(H0))*mean(diag(H0));
        H        = secular_H(H0,H1,specific_k);     
        [waveFnc,E]        = eigs(H,10,0.001);%2.97*sqrt(9/2*eta_B))
        E = diag(E);
        chosen_idx =1;
        x_lim = get(axes_geo,'xlim');
        %x_lim = [min(coordinates.x(:,1)) max(coordinates.x(:,1)) ];

        Position = [0.08 0.12 0.33 0.4];
        axes_wavefnc = axes('Parent',figure1, 'Position', Position,...
                'Visible', 'on',...
                'FontSize', fontsize,...   
                'xlim', x_lim,...                'ylim', y_lim,...     'XTick', XTick,...                'YTick', YTick,...
                'Box', 'on',...
                'FontName','Times New Roman');
        hold on; 

        square_Wavefnc = abs(waveFnc(1:length(coordinates.x(:,1)),chosen_idx)).^2;
        plot(coordinates.x(:,1), square_Wavefnc, 'MarkerSize', 4, 'Parent', axes_wavefnc) 
        
        % Create ylabel
        %ylabel_position = [2.5 0 0];
        ylabel('|\Psi|^2','FontSize', fontsize,'FontName','Times New Roman', 'Parent', axes_wavefnc);
        %ylabel_handle = get(axes_wavefnc,'YLabel'); 
        %set(ylabel_handle, 'Position', get(ylabel_handle, 'Position') + ylabel_position);

        chosen_k_pos = [ max(coordinates.x(:,1))*0.05, max(square_Wavefnc)*0.9];
        text(chosen_k_pos(1), chosen_k_pos(2), ['k_yr_{CC} = ',num2str(specific_k/sqrt(3),'% 10.2f')], 'FontSize', fontsize, 'FontName','Times New Roman',  'Parent', axes_wavefnc);
        E_pos = [ max(coordinates.x(:,1))*0.05, max(square_Wavefnc)*0.8];
        text(E_pos(1), E_pos(2), ['E = ',num2str(E(chosen_idx),'% 10.2f'),' eV'], 'FontSize', fontsize, 'FontName','Times New Roman',  'Parent', axes_wavefnc);
        B_pos = [ max(coordinates.x(:,1))*0.05, max(square_Wavefnc)*0.7];
        text(B_pos(1), B_pos(2), ['B = ',num2str(B,'% 10.2f'),' T'], 'FontSize', fontsize, 'FontName','Times New Roman',  'Parent', axes_wavefnc);        
        
        
        
%% Conductance   

        Position = [0.5 0.12 0.33 0.4];
        x_lim = [ min(Evec) max(Evec) ];
        axes_cond = axes('Parent',figure1, 'Position', Position,...
                'Visible', 'on',...
                'FontSize', fontsize,... 
                'xlim', x_lim,...                'ylim', y_lim,...                'XTick', XTick,...                'YTick', YTick,...
                'Box', 'on',...
                'FontName','Times New Roman');
        hold on; 
        grid on;
        
        indexek = logical( abs(DeltaC) <= 0.5 );
        
        numerics = plot(Evec(indexek), Conductance(indexek), 'Linewidth', 2, 'color', [0 0 0], 'Parent', axes_cond);
        %plot(Evec(indexek), nyitott_csatornak(indexek), 'Linewidth', 1, 'color', [1 0 0], 'Parent', axes_cond);
        try
            plot(Evec(indexek), DeltaC(indexek), 'Linewidth', 1, 'color', [0 1 0], 'Parent', axes_cond);
        catch
            disp('No DeltaC data present')
        end
        
        if ~isempty( hole_eigenenergies )
        	y_lim = [0 0.9];%[0 max(Conductance(indexek)) ];
        	for idx = 1:length(hole_eigenenergies)
        		plot( ones(1,2)*hole_eigenenergies(idx), y_lim, 'Linewidth', 1, 'color', [1 0 0], 'Parent', axes_cond);       	
        	end        
        end
               
        
        
        %plot(Evec, nyitott_csatornak, 'Linewidth', 1, 'color', [1 0 0], 'Parent', axes_cond);    
        
         % Create xlabel
        %xlabel_position = [0 0 0];
        xlabel('E [eV]','FontSize', fontsize,'FontName','Times New Roman', 'Parent', axes_cond);
        %xlabel_handle = get(axes_cond,'XLabel');  
        %set(xlabel_handle, 'Position', get(xlabel_handle, 'Position') + xlabel_position);

        % Create ylabel
        %ylabel_position = [0 0 0];
        ylabel('\sigma/\sigma_0','FontSize', fontsize,'FontName','Times New Roman', 'Parent', axes_cond);
        %ylabel_handle = get(axes_cond,'YLabel'); 
        %set(ylabel_handle, 'Position', get(ylabel_handle, 'Position') + ylabel_position);
        
        E_pos = [ max(Evec)*0.3, max(Conductance(indexek))*0.9];
        text(E_pos(1), E_pos(2), ['B = ',num2str(B,'% 10.2f'),' T, \phi/\phi_0 = ',num2str(create_hole.flux_of_hole,3)], 'FontSize', fontsize, 'FontName','Times New Roman',  'Parent', axes_cond);
        
        %set(get(get(numerics,'Annotation'),'LegendInformation'), 'IconDisplayStyle','off'); % Exclude line from legend

        fig_legend = legend(axes_cond, {'conductance', 'unitarity'});%, 'FontSize', fontsize, 'FontName','Times New Roman')
        set(fig_legend, 'FontSize', fontsize, 'FontName','Times New Roman', 'Box', 'off', 'Location', 'NorthEast')

        
        print('-depsc2', [outputdir,'/Hole_in_graphene_transport_', num2str(filenum),'.eps'])
        print('-dpdf', [outputdir,'/Hole_in_graphene_transport_', num2str(filenum),'.pdf'])
        if toCalculate
            close(figure1)
        end
        
        disp( 'Figure Created' )
        
        
    end




end
