%%    Eotvos Quantum Transport Utilities - FermiDirac
%    Copyright (C) 2016 Peter Rakyta, Ph.D.
%
%    This program is free software: you can redistribute it and/or modify
%    it under the terms of the GNU General Public License as published by
%    the Free Software Foundation, either version 3 of the License, or
%    (at your option) any later version.
%
%    This program is distributed in the hope that it will be useful,
%    but WITHOUT ANY WARRANTY; without even the implied warranty of
%    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%    GNU General Public License for more details.
%
%    You should have received a copy of the GNU General Public License
%    along with this program.  If not, see http://www.gnu.org/licenses/.
%
%
%   Returns a class describing the Fermi Dirac distribution of fermionic particles.
%%
classdef FermiDirac < handle %NEW
    
    properties (Access = protected)
        %> Boltzmann constant in eV/K
        k_B
        %> The temperature in Kelvin
        T = 0;        
        %> 1/(k_B*T)
        beta
        %> treshold temperature (makes difference between T=0 and T>0) NEW
        T_treshold = 1e-10;
    end
    
    
methods (Access=public)
        
    %% contructor of the class
    function obj = FermiDirac( varargin )        

        obj.k_B = 8.6173324e-5; % Boltzmann constant in eV/K
        obj.beta = [];

        obj.InputParsing( varargin{:});
        
        
    end
    
    
    %% Fermi distribution
    function ret = Fermi(obj, E)
        
        E = reshape(E, 1, numel(E));

		ret = zeros( length(obj.beta), length(E) );
        indexes = logical( abs(obj.T) <= obj.T_treshold);
        
        ret( indexes, E<0 ) = 1;
        ret( indexes ,E==0 ) = 0.5;
        ret( indexes, E>0)  = 0;
        
        if ~isempty(obj.beta(~indexes))
            ret( ~indexes, :) = 1./(1+exp(obj.beta(~indexes)*E));
        end
    end  


    %% setTemperature
	function setTemperature(obj, T)
        obj.T = reshape( T, numel(T), 1);
		obj.beta = 1./(obj.k_B*T); 
	end
    
    
    
end % public methods
    
    
methods (Access=private)
       
%% Input parsing
    function InputParsing( obj, varargin )
        
        
        p = inputParser;
        p.addParamValue('T', obj.T, @isscalar);
        
        p.parse(varargin{:});
       
        T               = p.Results.T;       
        
        obj.setTemperature(T); 
               

    end        
        
        
end % protected methods

    
end
