!======================================================================
!     Wrapper module to call the functions in the module hsx_m from the SIESTA package.
!     This file is part of the EQuUs package. (for details see: http://eqt.elte.hu/equus/home)
!     Copyright (C) 2016 Peter Rakyta, Ph.D.
!
!     This program is free software: you can redistribute it and/or modify
!     it under the terms of the GNU General Public License as published by
!     the Free Software Foundation, either version 3 of the License, or
!     (at your option) any later version.
! 
!     This program is distributed in the hope that it will be useful,
!     but WITHOUT ANY WARRANTY; without even the implied warranty of
!     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
!     GNU General Public License for more details.
!
!     You should have received a copy of the GNU General Public License
!     along with this program.  If not, see http://www.gnu.org/licenses/.
!     
!======================================================================
MODULE hsx_EQuUs

  use EQuUs_C
  use hsx_m

  type(hsx_t), public :: hsx



!****************************************************************      
  CONTAINS
!****************************************************************




!-------------------------------------------------------------------------------------------------
! DESCRIPTION:
!> @file hsx_EQuUs.f90
!> @brief read in the HSX structure into EQuUs structure prototype
!! 
!> @param[in] fname The name of the file to be loaded
!> @param[in] fname_len The length of the filename
!> @param[out] eq_struct An instance of EQuUs structure prototype
! 
  subroutine read_hsx( fname, fname_len, eq_struct)


  integer*8 fname_len
  character(len=fname_len), intent(in) :: fname
  logical :: file_exist

  TYPE(EQstruct) :: eq_struct
  TYPE(EQelement), target :: eq_elements(26)



! double check for the existence of the file 
  inquire(file=fname, exist=file_exist)
  if (.not.file_exist) then
    print *, 'File does not exist'
    stop
  endif


  ! Obtaining the HSX structure
  call read_hsx_file(  hsx, fname )


  call assocElement( eq_elements(1), "nspecies", hsx%nspecies )
  call assocElement( eq_elements(2), "na_u", hsx%na_u )
  call assocElement( eq_elements(3), "no_u", hsx%no_u )
  call assocElement( eq_elements(4), "no_s", hsx%no_s )
  call assocElement( eq_elements(5), "nspin", hsx%nspin )
  call assocElement( eq_elements(6), "nh", hsx%nh )
  call assocElement( eq_elements(7), "gamma", hsx%gamma )
  call assocElement( eq_elements(8), "has_xij", hsx%has_xij )
  call assocElement( eq_elements(9), "no", hsx%no )
  call assocElement( eq_elements(10), "nquant", hsx%nquant )
  call assocElement( eq_elements(11), "lquant", hsx%lquant )
  call assocElement( eq_elements(12), "zeta", hsx%zeta )
  call assocElement( eq_elements(13), "iaorb", hsx%iaorb )
  call assocElement( eq_elements(14), "iphorb", hsx%iphorb )
  call assocElement( eq_elements(15), "label", hsx%label )
  call assocElement( eq_elements(16), "numh", hsx%numh )
  call assocElement( eq_elements(17), "listhptr", hsx%listhptr )
  call assocElement( eq_elements(18), "listh", hsx%listh )
  call assocElement( eq_elements(19), "indxuo", hsx%indxuo )
  call assocElement( eq_elements(20), "hamilt", hsx%hamilt )
  call assocElement( eq_elements(21), "Sover", hsx%Sover )
  call assocElement( eq_elements(22), "xij", hsx%xij )
  call assocElement( eq_elements(23), "isa", hsx%isa )
  call assocElement( eq_elements(24), "zval", hsx%zval )
  call assocElement( eq_elements(25), "qtot", hsx%qtot )
  call assocElement( eq_elements(26), "temp", hsx%temp )
 

  eq_struct%fields = C_LOC(eq_elements(1))
  eq_struct%numfields = SIZE( eq_elements, 1 )


  end subroutine read_hsx




end module


