%    Automatic test procedure - based on EQuUs v4.7
%    Copyright (C) 2015 Peter Rakyta, Ph.D.
%
%    This program is free software: you can redistribute it and/or modify
%    it under the terms of the GNU General Public License as published by
%    the Free Software Foundation, either version 3 of the License, or
%    (at your option) any later version.
%
%    This program is distributed in the hope that it will be useful,
%    but WITHOUT ANY WARRANTY; without even the implied warranty of
%    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%    GNU General Public License for more details.
%
%    You should have received a copy of the GNU General Public License
%    along with this program.  If not, see http://www.gnu.org/licenses/.
function automatic_test()


% tests to run
fncnames = { 'magnetic_ribbon', ...
             'MinimalConductivity', ...
             'Graphene_Conductivity', ...
             'magnetic_barrier', ...
             'Differential_conductance', ...
             'Graphene_QAD_eig', ...
             'Graphene_antidot_transport', ...
             'JosephsonCurrent', ...
             'SpectralDensity', ...
             'SSH_transport', ...
             'SSH_Josephson' ...
             };
         
Dirs     = { 'magnetic_ribbon', ...
             'Graphene_Minimal_Conductivity', ...
             'Graphene_Conductivity', ...
             'magnetic_barrier_0B0', ...
             'Differential_conductance', ...
             'Graphene_antidot_eigenstate', ...
             'Graphene_antidot_transport', ...
             'SNS_Josephson', ...
             'SpectralDensity', ...
             'SSH_transport', ...
             'SSH_Josephson' ...
             };
             
             

filename = mfilename('fullpath');
[root, fncname] = fileparts( filename );
cd(root)

test_results = false(size(fncnames));

for jdx = 1:length(fncnames)
    test_results(jdx) = runTest( fncnames{jdx}, Dirs{jdx});
end




if norm( true(size(test_results)) - test_results ) <= 1e-6
    disp('*************************');
    disp('*************************');
    disp('All tests passed.')
    disp('*************************');
else
    for jdx = 1:length(fncnames)
        if test_results( jdx )
            continue
        end
        failed_fncname = fncnames{ jdx }; 
        failed_Dir = Dirs{jdx};
        disp(['Test ', fullfile(root, failed_Dir, failed_fncname), ' failed.']);
    end
end





% run the test
    function ret = runTest( fncname, directory)
        %try
            directory = fullfile( root, 'Examples', directory);
            filenum = getFilenum( directory );
            cd(directory)
            feval( fncname, filenum );
            cd(root)
            ret = true;
        %catch err
        %    ret = false;
        %    rethrow(err)
        %end
            
    end



% determine the next filename
    function ret = getFilenum( directory )
        ret = 1;
        
        numerics = {'1' '2' '3' '4' '5' '6' '7' '8' '9' '0'};
        
        start_idx = [];
        end_idx = [];
        
        try
            list = dir( fullfile( directory, 'results', '*.mat') );
        catch err
            ret = 1;
            return
        end
        
        if isempty( list )
            ret = 1;
            return
        end
        
        
        for kdx = 1:length(list)
            filename_tmp = list(kdx).name;
            for idx = 1:length(filename_tmp)
                character = filename_tmp(idx);
                comp = find(strcmp( character, numerics ));
            
                if norm(comp) > 0
                    if isempty( start_idx )
                        start_idx = idx;
                    end
                else
                    if isempty( end_idx ) && ~isempty( start_idx )
                        end_idx = idx;
                    end
                end
            
            
                if ~isempty( start_idx ) && ~isempty(end_idx )
                    number_tmp = str2double( filename_tmp(start_idx:end_idx-1) );
                    if ret < number_tmp
                        ret = number_tmp;
                    end
                    start_idx = [];
                    end_idx   = [];
                end
            
            
            end
        end
        
        ret = ret+1;
        
        
        
    end


end
